SystemOrganization addCategory: #LostInMorvanChat!
SystemOrganization addCategory: 'LostInMorvanChat-Client'!
SystemOrganization addCategory: 'LostInMorvanChat-Message'!
SystemOrganization addCategory: 'LostInMorvanChat-Server'!
SystemOrganization addCategory: 'LostInMorvanChat-User'!

ComposableModel subclass: #ChatConsole
	instanceVariableNames: 'input chatClient messagesList usersList'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Client'!

!ChatConsole commentStamp: 'FrancoisJarriges 6/16/2016 17:20' prior: 0!
I represent the console window to be used for the chat application to write and read messages.
 
Internal Representation and Key Implementation Points.

    Instance Variables
	chatClient:		<ChatClient>
	input:			<TextInputFieldModel>
	messagesList:	<ChatConsoleListModel>
	usersList: 		<LisModel>


    Implementation Points!

!ChatConsole class methodsFor: 'instance-creation' stamp: 'FrancoisJarriges 6/14/2016 14:16'!
attach: aChatClient

	| aChatConsole |
	
	aChatConsole := self new chatClient: aChatClient.
	aChatConsole openWithSpec whenClosedDo: [ aChatClient disconnect ].
	^ aChatConsole! !

!ChatConsole class methodsFor: 'api' stamp: 'FrancoisJarriges 6/16/2016 18:32'!
defaultSpec
	<spec: #default>
	
	^ SpecLayout composed
			newColumn: [ :column | column add: #usersList ]
			width: 140;
			newColumn: [ :column |
				column add: #messagesList; add: #input height: 30 ]
			left: 150;
			yourself
! !

!ChatConsole methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/16/2016 17:35'!
chatClient: aChatClient

	chatClient := aChatClient! !

!ChatConsole methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/16/2016 17:34'!
initializeWidgets

	usersList := ListModel new.
	messagesList := ChatConsoleListModel new.
	input := TextInputFieldModel new
		ghostText: 'Type your message here...';
		enabled: true;
		acceptBlock: [ :inputString |
			chatClient send: inputString.
			input text: '' ].
	self focusOrder add: input
! !

!ChatConsole methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/16/2016 17:25'!
input

	^ input! !

!ChatConsole methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/16/2016 17:24'!
messagesList

	^ messagesList! !

!ChatConsole methodsFor: 'api' stamp: 'FrancoisJarriges 6/16/2016 17:28'!
printMessages: aCollectionOfChatMessages

	messagesList addItems: aCollectionOfChatMessages 
! !

!ChatConsole methodsFor: 'api' stamp: 'FrancoisJarriges 6/16/2016 17:34'!
printUsers: aCollectionOfByteStrings

	usersList items: aCollectionOfByteStrings
! !

!ChatConsole methodsFor: 'api' stamp: 'FrancoisJarriges 6/14/2016 14:16'!
title
	^ chatClient login, ' on ', chatClient url! !

!ChatConsole methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/16/2016 17:24'!
usersList

	^ usersList! !

ListModel subclass: #ChatConsoleListModel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Client'!

!ChatConsoleListModel commentStamp: 'FrancoisJarriges 6/2/2016 15:06' prior: 0!
I specialize the standard UI ListModel class for a chat console window.


    Implementation Points!

!ChatConsoleListModel methodsFor: 'api' stamp: 'FrancoisJarriges 6/2/2016 12:55'!
addItems: aList
	"Add a new list of items to the existing list."

	| oldContents |
	oldContents := listHolder value.
	listHolder value: {oldContents . (aList sorted: sortingBlockHolder value)} concatenation.
	listAnnouncer announce: (ValueChanged oldValue: oldContents newValue: listHolder value).
	! !

Announcement subclass: #ChatMessageAnnouncement
	instanceVariableNames: 'message'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Server'!

!ChatMessageAnnouncement commentStamp: 'FrancoisJarriges 6/3/2016 19:41' prior: 0!
I represent a new message  event  raised from a server thread at each reception of a new message post.
 
Internal Representation and Key Implementation Points.

    Instance Variables
	message:		<ChatMessage>


    Implementation Points!

!ChatMessageAnnouncement class methodsFor: 'instance-creation' stamp: 'FrancoisJarriges 6/3/2016 19:54'!
for: aChatMessage
	
	 ^ self new message: aChatMessage! !

!ChatMessageAnnouncement methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/3/2016 20:34'!
message

	^ message! !

!ChatMessageAnnouncement methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/3/2016 19:55'!
message: aChatMessage

	message := aChatMessage! !

Object subclass: #ChatClient
	instanceVariableNames: 'url login exit messages console lastMessageIndex requests lastErrorString'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Client'!

!ChatClient commentStamp: 'FrancoisJarriges 6/16/2016 16:58' prior: 0!
I am a chat client. I'm sending my own messages  to an HTTP server and getting all the messages sent by all the chat clients including me to display on a console. 

ChatClient connect: 'localhost' port: 8080 login: 'toto'.
ChatClient connect: 'eph-30278de7.swarm.pharocloud.com' login: 'toto'
 
Internal Representation and Key Implementation Points.

    Instance Variables
	url:						<ByteString>
	login:					<ByteString>
	exit:						<Boolean>
	messages:				<OrderedCollection>
	console:					<ChatConsole>
	lastMessageIndex:		<Integer>
	lastErrorString:			<ByteString>
	requests: 				<ChatClientRequests>
	

    Implementation Points!

!ChatClient class methodsFor: 'public' stamp: 'FrancoisJarriges 6/15/2016 09:09'!
connect: aHost login: aLogin

	^ self new
		host: aHost login: aLogin;
		start
! !

!ChatClient class methodsFor: 'public' stamp: 'FrancoisJarriges 6/1/2016 11:19'!
connect: aHost port: aPort login: aLogin

	^ self new
		host: aHost port: aPort login: aLogin;
		start
! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/16/2016 15:11'!
chatMessage: aByteString
	"Chat messages are actual messages from chatting users"

	^ ChatMessage from: login text: aByteString! !

!ChatClient methodsFor: 'api' stamp: 'FrancoisJarriges 6/16/2016 16:48'!
disconnect
	"Here sending logout message is delegated to a light process in order to free the console immediately."

	exit ifFalse: [ 
		exit := true.
		[ self 
			http: requests putLogout 
			with: self logStamp ] fork ].
	^ self


! !

!ChatClient methodsFor: 'api' stamp: 'FrancoisJarriges 6/16/2016 18:37'!
getConnectedUsers
	"Get all alive users from the server and display their login names"
	
	console printUsers: (self http: requests getConnectedUsers)

! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/16/2016 17:28'!
getNewMessages
	"Gets the new messages that have been posted since the last request."
	"Prints the new messages on the console."

	| receivedChatMessages |
	
	receivedChatMessages := self http: requests getMessagesFromLastIndex.
	console printMessages: (receivedChatMessages select: [ :eachChatMessage | 
		eachChatMessage sender ~= login ]).
	messages addAll: receivedChatMessages.
	^ receivedChatMessages size
	
! !

!ChatClient methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/15/2016 09:09'!
host: aHost login: aLogin

	url := 'http://' , aHost.
	login := aLogin

! !

!ChatClient methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/1/2016 11:22'!
host: aHost port: aPort login: aLogin

	url := 'http://' , aHost , ':' , aPort asString.
	login := aLogin

! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/16/2016 14:46'!
http: aChatClientGet

	^ ((STONReader on: (aChatClientGet basicRequest) readStream) next)! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/16/2016 16:35'!
http: aChatClientPut with: aChatMessage

	aChatClientPut basicRequest: aChatMessage! !

!ChatClient methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/16/2016 16:58'!
initialize

	super initialize.
	exit := false.
	lastErrorString := ''.
	lastMessageIndex := 0.
	requests := ChatClientRequests attach: self.
	messages := OrderedCollection new! !

!ChatClient methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/16/2016 14:56'!
lastMessageIndex

	^ lastMessageIndex ! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/16/2016 15:18'!
logStamp
	"basic service message used at login/out time"
	
	^ ChatMessage from: login text: ''! !

!ChatClient methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/16/2016 14:57'!
login

	^ login! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/16/2016 17:28'!
printConnectionErrorMessage: aNetworkErrorString

	| errorMessage |
	
	aNetworkErrorString = lastErrorString ifFalse: [ 
		lastErrorString := aNetworkErrorString.
		errorMessage := self serviceMessage: aNetworkErrorString.
		console printMessages: errorMessage asOrderedCollection ].
	^ self


	! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/16/2016 18:39'!
refreshMessages
	"Create and schedule a light process to get new posted messages every 2 seconds."
	"By the way collect the list of connected users as well."
	"At client disconnection the process ends."
	
	[ 
		[ exit ] whileFalse: [
			lastMessageIndex := lastMessageIndex + self getNewMessages.
			self getConnectedUsers.
			(Delay forSeconds: 2) wait ] 
	] fork! !

!ChatClient methodsFor: 'api' stamp: 'FrancoisJarriges 6/16/2016 17:28'!
send: aByteString
	"Get the string input from the console and print it back to it."
	"Post a message to the server which text is this string."
	
	| newChatMessage |
	
	newChatMessage := self chatMessage: aByteString.
	console printMessages: (OrderedCollection with: newChatMessage).
	self http: requests putMessage with: newChatMessage 
! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/16/2016 15:11'!
serviceMessage: aByteString
	"Service messages are internally built and display information or error data"

	^ ChatMessage from: '-----' text: aByteString! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/16/2016 16:35'!
start

	console := ChatConsole attach: self.
	lastMessageIndex := self http: requests getLastMessageIndex.
	self http: requests putLogin with: self logStamp.
	self refreshMessages! !

!ChatClient methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/2/2016 17:04'!
url
	^ url! !

Object subclass: #ChatClientHttpRequest
	instanceVariableNames: 'command userId chatClient'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Client'!

ChatClientHttpRequest class
	instanceVariableNames: 'path'!

!ChatClientHttpRequest commentStamp: 'FrancoisJarriges 6/16/2016 15:57' prior: 0!
I represent an abstract class of an HTTP request sent by this  client chat application.
 
Internal Representation and Key Implementation Points.

    Instance Variables
	chatClient:		<ChatClient>
	command:		<ByteString>
	userId:			<ByteString>
				
    Class Instance Variables 
	path:			<ByteString>


    Implementation Points!

ChatClientHttpRequest class
	instanceVariableNames: 'path'!

ChatClientHttpRequest subclass: #ChatClientGet
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Client'!

!ChatClientGet commentStamp: 'FrancoisJarriges 6/14/2016 15:19' prior: 0!
I represent an abstract class of all HTTP GET requests sent by this  chat client.

Internal Representation and Key Implementation Points.


    Implementation Points!

!ChatClientGet methodsFor: 'api' stamp: 'FrancoisJarriges 6/14/2016 14:22'!
basicRequest
		
	| responseString |
	
	[ responseString := ZnClient new 
		url: self command; 
		headerAt: 'ChatUserId' put: self userId;
		get ] 
		on: NetworkError 
		do: [ :thisNetworkError | 
			responseString := self handleNetWorkError: thisNetworkError ].
	^ responseString ! !

!ChatClientGet methodsFor: 'private' stamp: 'FrancoisJarriges 6/14/2016 14:28'!
handleNetWorkError: aNetworkError
	
	chatClient printConnectionErrorMessage: aNetworkError asString.
	^ STON toString: OrderedCollection new! !

ChatClientGet subclass: #ChatClientGetConnectedUsers
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Client'!

!ChatClientGetConnectedUsers commentStamp: 'FrancoisJarriges 6/14/2016 15:19' prior: 0!
I represent an HTTP request sent by this chat client to get all connected users from the server 
 
Internal Representation and Key Implementation Points.


    Implementation Points!

!ChatClientGetConnectedUsers class methodsFor: 'class initialization' stamp: 'FrancoisJarriges 6/14/2016 16:18'!
initialize

	path := '/users/connected'! !

ChatClientGetConnectedUsers initialize!

ChatClientGet subclass: #ChatClientGetLastMessageIndex
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Client'!

!ChatClientGetLastMessageIndex commentStamp: 'FrancoisJarriges 6/14/2016 15:19' prior: 0!
I represent an HTTP request sent by this chat client to get the last message index from the server.
 
Internal Representation and Key Implementation Points.


    Implementation Points!

!ChatClientGetLastMessageIndex class methodsFor: 'class initialization' stamp: 'FrancoisJarriges 6/14/2016 16:19'!
initialize
	
		path := '/messages/count'! !

ChatClientGetLastMessageIndex initialize!

!ChatClientGetLastMessageIndex methodsFor: 'private' stamp: 'FrancoisJarriges 6/14/2016 14:27'!
handleNetWorkError: aNetworkError
	
	chatClient printConnectionErrorMessage: aNetworkError asString.
	^ STON toString: chatClient lastMessageIndex ! !

ChatClientGet subclass: #ChatClientGetMessagesFromLastIndex
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Client'!

!ChatClientGetMessagesFromLastIndex commentStamp: 'FrancoisJarriges 6/14/2016 15:19' prior: 0!
I represent an HTTP  request sent by this chat client to get  all  the last messages posted to the server.
 
Internal Representation and Key Implementation Points.


    Implementation Points!

!ChatClientGetMessagesFromLastIndex class methodsFor: 'class initialization' stamp: 'FrancoisJarriges 6/15/2016 19:31'!
initialize
	
	path := '/messages/from'! !

ChatClientGetMessagesFromLastIndex initialize!

!ChatClientGetMessagesFromLastIndex methodsFor: 'api' stamp: 'FrancoisJarriges 6/15/2016 19:38'!
basicRequest
		
	| responseString |
	
	[ responseString := ZnClient new 
		url: self command; 
		headerAt: 'ChatUserId' put: self userId;
		headerAt: 'LastMessageIndex' put: chatClient lastMessageIndex asString;
		get ] 
		on: NetworkError 
		do: [ :thisNetworkError | 
			responseString := self handleNetWorkError: thisNetworkError ].
	^ responseString ! !

!ChatClientHttpRequest class methodsFor: 'instance-creation' stamp: 'FrancoisJarriges 6/16/2016 14:05'!
attach: aChatClient

	path ifNil: [ self initialize ].
	^ self new chatClient: aChatClient; yourself! !

!ChatClientHttpRequest class methodsFor: 'private' stamp: 'FrancoisJarriges 6/14/2016 16:17'!
path

	^ path! !

!ChatClientHttpRequest methodsFor: 'api' stamp: 'FrancoisJarriges 6/14/2016 13:14'!
basicRequest

	^ self subclassResponsibility ! !

!ChatClientHttpRequest methodsFor: 'api' stamp: 'FrancoisJarriges 6/14/2016 13:41'!
basicRequest: anObject

	^ self subclassResponsibility ! !

!ChatClientHttpRequest methodsFor: 'private' stamp: 'FrancoisJarriges 6/14/2016 14:15'!
chatClient: aChatClient

	chatClient := aChatClient 
	! !

!ChatClientHttpRequest methodsFor: 'private' stamp: 'FrancoisJarriges 6/14/2016 16:19'!
command

	^ command ifNil: [ command := '{1}{2}' 
			format: { chatClient url . self class path } ]! !

!ChatClientHttpRequest methodsFor: 'private' stamp: 'FrancoisJarriges 6/14/2016 14:05'!
handleNetWorkError: aNetworkError

	^ self subclassResponsibility ! !

!ChatClientHttpRequest methodsFor: 'private' stamp: 'FrancoisJarriges 6/14/2016 14:19'!
userId

	^ userId ifNil: [ userId := chatClient login asCamelCase uncapitalized ]
	! !

ChatClientHttpRequest subclass: #ChatClientPut
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Client'!

!ChatClientPut commentStamp: 'FrancoisJarriges 6/15/2016 14:09' prior: 0!
I represent an abstract class of all HTTP PUT requests sent by this  chat client.

 
Internal Representation and Key Implementation Points.


    Implementation Points!

!ChatClientPut methodsFor: 'api' stamp: 'FrancoisJarriges 6/15/2016 19:26'!
basicRequest: aChatMessage

	[ ZnClient new
		url: self command;
		headerAt: 'ChatUserId' put: self userId;
		entity: (ZnEntity 
              with: (STON toString: aChatMessage)
              type: ZnMimeType applicationSton);
		put ]
		on: NetworkError 
		do: [ :thisNetworkError | 
			self handleNetWorkError: thisNetworkError ]! !

!ChatClientPut methodsFor: 'private' stamp: 'FrancoisJarriges 6/15/2016 07:50'!
handleNetWorkError: aNetworkError
	
	chatClient printConnectionErrorMessage: aNetworkError asString! !

ChatClientPut subclass: #ChatClientPutLogin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Client'!

!ChatClientPutLogin commentStamp: 'FrancoisJarriges 6/15/2016 14:05' prior: 0!
I represent an HTTP  request sent by this chat client to signal user login to the server.

 
Internal Representation and Key Implementation Points.


    Implementation Points!

!ChatClientPutLogin class methodsFor: 'class initialization' stamp: 'FrancoisJarriges 6/15/2016 10:55'!
initialize
	
	path := '/users/login'! !

ChatClientPutLogin initialize!

ChatClientPut subclass: #ChatClientPutLogout
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Client'!

!ChatClientPutLogout commentStamp: 'FrancoisJarriges 6/15/2016 14:05' prior: 0!
I represent an HTTP  request sent by this chat client to signal user logout to the server.
 
 
Internal Representation and Key Implementation Points.


    Implementation Points!

!ChatClientPutLogout class methodsFor: 'class initialization' stamp: 'FrancoisJarriges 6/15/2016 07:55'!
initialize
	
	path := '/users/logout'! !

ChatClientPutLogout initialize!

ChatClientPut subclass: #ChatClientPutMessage
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Client'!

!ChatClientPutMessage commentStamp: 'FrancoisJarriges 6/15/2016 14:09' prior: 0!
I represent an HTTP  request sent by this chat client to push  a message to the server.
 
Internal Representation and Key Implementation Points.


    Implementation Points!

!ChatClientPutMessage class methodsFor: 'class initialization' stamp: 'FrancoisJarriges 6/14/2016 16:19'!
initialize
	
	path := '/messages/add'! !

ChatClientPutMessage initialize!

Object subclass: #ChatClientRequests
	instanceVariableNames: 'chatClient getConnectedUsers getLastMessageIndex getMessagesFromLastIndex putLogin putLogout putMessage'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Client'!

!ChatClientRequests commentStamp: 'FrancoisJarriges 6/16/2016 16:59' prior: 0!
I represent the set of all HTTP requests sent by this  client chat application.
 
Internal Representation and Key Implementation Points.

    Instance Variables

	chatClient: 					<ChatClient>
	getConnectedUsers:			<ChatClientGetConnectedUsers>
	getLastMessageIndex:			<ChatClientGetLastMessageIndex>
	getMessagesFromLastIndex:  	<ChatClientGetMessagesFromLastIndex>
	putMessage:					<ChatClientPutMessage>
	putLogIn:					<ChatClientPutLogin>
	putLogout: 					<ChatClientPutLogout>


    Implementation Points!

!ChatClientRequests class methodsFor: 'instance-creation' stamp: 'FrancoisJarriges 6/16/2016 16:18'!
attach: aChatClient

	^ self new 
		chatClient: aChatClient; 
		getConnectedUsers: 
			(ChatClientGetConnectedUsers attach: aChatClient);
		getLastMessageIndex: 
			(ChatClientGetLastMessageIndex attach: aChatClient);
		getMessagesFromLastIndex: 
			(ChatClientGetMessagesFromLastIndex attach: aChatClient);
		putLogin: 
			(ChatClientPutLogin attach: aChatClient);
		putLogout: 
			(ChatClientPutLogout attach: aChatClient);
		putMessage: 
			(ChatClientPutMessage attach: aChatClient);
		yourself! !

!ChatClientRequests methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/16/2016 16:14'!
chatClient
	^ chatClient! !

!ChatClientRequests methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/16/2016 16:14'!
chatClient: aChatClient
	chatClient := aChatClient! !

!ChatClientRequests methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/16/2016 16:08'!
getConnectedUsers
	^ getConnectedUsers! !

!ChatClientRequests methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/16/2016 16:10'!
getConnectedUsers: aChatClientGetConnectedUsers
	getConnectedUsers := aChatClientGetConnectedUsers! !

!ChatClientRequests methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/16/2016 16:09'!
getLastMessageIndex
	^ getLastMessageIndex! !

!ChatClientRequests methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/16/2016 16:10'!
getLastMessageIndex: aChatClientGetLastMessageIndex
	getLastMessageIndex := aChatClientGetLastMessageIndex! !

!ChatClientRequests methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/16/2016 16:09'!
getMessagesFromLastIndex
	^ getMessagesFromLastIndex! !

!ChatClientRequests methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/16/2016 16:11'!
getMessagesFromLastIndex: aChatClientGetMessagesFromLastIndex
	getMessagesFromLastIndex := aChatClientGetMessagesFromLastIndex! !

!ChatClientRequests methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/16/2016 16:05'!
initialize

	getConnectedUsers := ChatClientGetConnectedUsers attach: self.
	getLastMessageIndex := ChatClientGetLastMessageIndex attach: self.
	getMessagesFromLastIndex := ChatClientGetMessagesFromLastIndex attach: self.
	putMessage := ChatClientPutMessage attach: self.
	putLogin := ChatClientPutLogin attach: self.
	putLogout :=  ChatClientPutLogout attach: self.! !

!ChatClientRequests methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/16/2016 16:09'!
putLogin
	^ putLogin! !

!ChatClientRequests methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/16/2016 16:11'!
putLogin: aChatClientPutLogin
	putLogin := aChatClientPutLogin! !

!ChatClientRequests methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/16/2016 16:09'!
putLogout
	^ putLogout! !

!ChatClientRequests methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/16/2016 16:11'!
putLogout: aChatClientPutLogout
	putLogout := aChatClientPutLogout! !

!ChatClientRequests methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/16/2016 16:09'!
putMessage
	^ putMessage! !

!ChatClientRequests methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/16/2016 16:12'!
putMessage: aChatClientPutMessage
	putMessage := aChatClientPutMessage! !

Object subclass: #ChatMessage
	instanceVariableNames: 'sender text'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Message'!

!ChatMessage commentStamp: 'FrancoisJarriges 6/2/2016 14:57' prior: 0!
I'm a simple chat message. I identify my sender and my text.
 
Internal Representation and Key Implementation Points.

    Instance Variables
	sender:		<Object>
	text:		<Object>


    Implementation Points!

!ChatMessage class methodsFor: 'instance-creation' stamp: 'FrancoisJarriges 6/1/2016 08:59'!
from: aSender text: aText

	^ self new sender: aSender; text: aText; yourself
! !

!ChatMessage class methodsFor: 'api' stamp: 'FrancoisJarriges 6/1/2016 12:54'!
separator
	^ '>'! !

!ChatMessage methodsFor: 'printing' stamp: 'FrancoisJarriges 6/1/2016 15:01'!
printOn: aStream

	aStream 
		<< self sender; 
		<< self class separator;
		<< String space;
		<< self text; 
		<< String crlf! !

!ChatMessage methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/1/2016 08:57'!
sender
	^ sender! !

!ChatMessage methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/4/2016 08:10'!
sender: aByteString
	sender := aByteString! !

!ChatMessage methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/1/2016 08:57'!
text
	^ text! !

!ChatMessage methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/4/2016 08:10'!
text: aByteString
	text := aByteString! !

Object subclass: #ChatMessagesQueue
	instanceVariableNames: 'messages announcer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Server'!

!ChatMessagesQueue commentStamp: 'FrancoisJarriges 6/3/2016 19:46' prior: 0!
I represent a list of all the messages sent by clients in this chat application.

    Instance Variables
	messages:		<OrderedCollection>
	announcer: 		<Announcer>


    Implementation Points!

!ChatMessagesQueue methodsFor: 'adding' stamp: 'FrancoisJarriges 6/4/2016 03:59'!
add: aChatMessage
	messages add: aChatMessage 
! !

!ChatMessagesQueue methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/3/2016 19:46'!
announcer
	
	^ announcer ifNil: [ announcer := Announcer new ]! !

!ChatMessagesQueue methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/4/2016 04:33'!
initialize

	super initialize.
	messages := OrderedCollection new.
	self announcer 
		when: ChatMessageAnnouncement 
		do: [ :thisAnnouncement |
			self add: thisAnnouncement message ]
 ! !

!ChatMessagesQueue methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/15/2016 15:23'!
listFrom: anIndex

	^ (anIndex between: 0 and: messages size - 1)
		ifTrue: [ messages copyFrom: anIndex + 1 to: messages size ]
		ifFalse: [ OrderedCollection new ]! !

!ChatMessagesQueue methodsFor: 'resetting' stamp: 'FrancoisJarriges 6/4/2016 04:13'!
reset
	self announcer unsubscribe: self.
	messages removeAll! !

!ChatMessagesQueue methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/1/2016 09:12'!
size
	^ messages size! !

Object subclass: #ChatServer
	instanceVariableNames: 'teapotServer messagesQueue log'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Server'!

!ChatServer commentStamp: 'FrancoisJarriges 6/14/2016 15:38' prior: 0!
I'm a chat server. I'm listening to HTTPRequests and managing a list of messages. 

My clients often request the messages I hold. Indeed, it may happen that client sends a lot of message and another one should request the difference between the messages it knows and the ones published on the server to be able to refresh correctly. 

I register some routes 
	/messages/ aNumber  get all the messages from aNumber to the end.
	/messages/count to get the current number of messages
	/messages/ add to add a message

ChatServer stopAll.
ChatServer startOn: 8080.
 
Internal Representation and Key Implementation Points.

    Instance Variables
	teapotServer:		<Teapot>
	messagesQueue:		<ChatMessagesQueue>
	log					<ChatUsersLogged>


    Implementation Points
	As  my underlying ZnServer could spawn child threads to cope with concurrent POST requests:
	- in the context of a POST request treatment  a ChatMessageAnnoucement is made that communicates the message content to the ChatMessagesQueue announcer 
	- the actual writing operation of this new message to the ChatMessagesQueue is then realized in the context of the main server thread by the ChatMessagesQueue itself (that has subscribed to the ChatMessageAnnouncement at start)!

!ChatServer class methodsFor: 'public' stamp: 'FrancoisJarriges 6/3/2016 15:01'!
startOn: aPortNumber

	^ self new
		initializePort: aPortNumber;
		registerRoutes;
		registerErrorHandlers;
		start;
		yourself! !

!ChatServer class methodsFor: 'image-startup' stamp: 'FrancoisJarriges 6/16/2016 16:52'!
startup
	self class stopAll.
	self class startOn: 8080! !

!ChatServer class methodsFor: 'public' stamp: 'FrancoisJarriges 6/1/2016 09:44'!
stopAll

	^ self allInstancesDo: #stop! !

!ChatServer methodsFor: 'requests' stamp: 'FrancoisJarriges 6/15/2016 10:52'!
addMessage: aTeaRequest

	| newChatMessage |
	
	newChatMessage := (STONReader on: aTeaRequest entity string readStream) next.

	messagesQueue announcer 
		announce: (ChatMessageAnnouncement for: newChatMessage)
! !

!ChatServer methodsFor: 'requests' stamp: 'FrancoisJarriges 6/5/2016 10:23'!
connectedUsers

	^ log users values 
		select: [ :eachUser | eachUser connected ]
		thenCollect: [ :eachConnectedUser | eachConnectedUser login ]
! !

!ChatServer methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/4/2016 22:13'!
initialize

	super initialize.
	messagesQueue := ChatMessagesQueue new.
	log := ChatUsersLogged  new
! !

!ChatServer methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/3/2016 15:25'!
initializePort: anInteger

	teapotServer := Teapot configure: {
		#defaultOutput -> #text.
		#port -> anInteger.
		#debugMode -> true }
! !

!ChatServer methodsFor: 'requests' stamp: 'FrancoisJarriges 6/4/2016 23:45'!
messageCount

	^ messagesQueue size! !

!ChatServer methodsFor: 'requests' stamp: 'FrancoisJarriges 6/15/2016 19:39'!
messagesFrom: aTeaRequest

	| thisChatUserId lastMessageIndex | 

	thisChatUserId := aTeaRequest headers at: 'ChatUserId'.
	lastMessageIndex := aTeaRequest headers at: 'LastMessageIndex'.
	(log users at: thisChatUserId asSymbol) lifeSign: true.
	^ messagesQueue listFrom: lastMessageIndex asInteger ! !

!ChatServer methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/15/2016 17:27'!
registerErrorHandlers
	"to be implemented in a different way"
	"teapotServer 
		exception: KeyNotFound -> (TeaResponse notFound body: 'No such message')"! !

!ChatServer methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/15/2016 19:37'!
registerRoutes
	
	teapotServer 
		GET: '/users/connected' -> (Send message: #connectedUsers to: self); output: #ston;
		GET: '/messages/count' -> (Send message: #messageCount to: self); output: #ston;
		GET: '/messages/from' -> (Send message: #messagesFrom: to: self); output: #ston;
		PUT: '/messages/add' -> (Send message: #addMessage: to: self);
		PUT: 'users/login' -> (Send message: #userLogin: to: self);
		PUT: '/users/logout' -> (Send message: #userLogout: to: self)! !

!ChatServer methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/4/2016 23:49'!
start
		
	teapotServer start.
	log updateConnections 
	

! !

!ChatServer methodsFor: 'public' stamp: 'FrancoisJarriges 6/4/2016 23:50'!
stop
	
	teapotServer stop.
	log stopUpdateConnections.
	messagesQueue reset! !

!ChatServer methodsFor: 'requests' stamp: 'FrancoisJarriges 6/15/2016 10:38'!
userLogin: aTeaRequest

	| loginChatMessage thisChatUserId |
	
	loginChatMessage := (STONReader on: aTeaRequest entity string readStream) next.
	thisChatUserId := aTeaRequest headers at: 'ChatUserId'.
	log users at: thisChatUserId asSymbol 
		ifAbsentPut: [ ChatUser withLogin: loginChatMessage sender].

	messagesQueue announcer 
		announce: (ChatMessageAnnouncement for: (ChatMessage 
			from: '-----'
			text: loginChatMessage sender, ' joined the chat room'))
! !

!ChatServer methodsFor: 'requests' stamp: 'FrancoisJarriges 6/15/2016 16:57'!
userLogout: aTeaRequest

	| thisChatUserId thisUser |
	
	thisChatUserId := aTeaRequest headers at: 'ChatUserId'.
	thisUser := (log users at: thisChatUserId asSymbol)
		lifeSign: false;
		connected: false.
	messagesQueue announcer 
		announce: (ChatMessageAnnouncement for: (ChatMessage 
			from: '-----'
			text: thisUser login, ' exited from the chat room'))
! !

Object subclass: #ChatUser
	instanceVariableNames: 'login connected lifeSign'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-User'!

!ChatUser commentStamp: 'FrancoisJarriges 6/4/2016 17:31' prior: 0!
I represent a user in of this tiny chat application.
 
Internal Representation and Key Implementation Points.

    Instance Variables

	login:		<ByteString>


    Implementation Points!

!ChatUser class methodsFor: 'instance-creation' stamp: 'FrancoisJarriges 6/4/2016 22:06'!
withLogin: loginString
	
	^ self new 
		login: loginString;
		connected: true;
		lifeSign: true;
		yourself! !

!ChatUser methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/16/2016 16:54'!
connected

	^ connected ifNil: [ connected := false ]! !

!ChatUser methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/16/2016 16:54'!
connected: aBoolean

	connected := aBoolean! !

!ChatUser methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/16/2016 16:54'!
lifeSign

	^ lifeSign ifNil: [ lifeSign := false ]! !

!ChatUser methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/16/2016 16:54'!
lifeSign: aBoolean

	lifeSign := aBoolean! !

!ChatUser methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/16/2016 16:54'!
login

	^ login ifNil: [ login := '' ]! !

!ChatUser methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/16/2016 16:54'!
login: aByteString

	login := aByteString! !

!ChatUser methodsFor: 'printing' stamp: 'FrancoisJarriges 6/4/2016 17:31'!
printOn: aStream

	aStream 
		<< self login; 
		<< String crlf! !

Object subclass: #ChatUsersLogged
	instanceVariableNames: 'users checkConnections'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Server'!

!ChatUsersLogged commentStamp: 'FrancoisJarriges 6/4/2016 22:09' prior: 0!
I represent all the users connections to the chat since the server started.
 
Internal Representation and Key Implementation Points.

    Instance Variables
	users:				<Dictionary>
	checkConnections: 	<Boolean>


    Implementation Points
	!

!ChatUsersLogged methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/4/2016 22:21'!
initialize

	super initialize.
	users := Dictionary new.
	checkConnections := true! !

!ChatUsersLogged methodsFor: 'private' stamp: 'FrancoisJarriges 6/16/2016 16:54'!
stopUpdateConnections

	checkConnections := false! !

!ChatUsersLogged methodsFor: 'private' stamp: 'FrancoisJarriges 6/5/2016 11:20'!
updateConnections
	"Create and schedule a light process to check users connections live signs every 15 seconds."
	"When the server stops the process ends."
	
	[
		[ checkConnections ] whileTrue: [
			(Delay forSeconds: 15) wait.
			 users do: [ :eachUser | 
				eachUser connected: eachUser lifeSign.
				eachUser lifeSign: false ] ]
	] fork! !

!ChatUsersLogged methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/16/2016 16:54'!
users

	^ users! !

ChatClientPutMessage initialize!

ChatClientPutLogout initialize!

ChatClientGetLastMessageIndex initialize!

ChatClientPutLogin initialize!

ChatClientGetMessagesFromLastIndex initialize!

ChatClientGetConnectedUsers initialize!
